@@ -9,7 +9,7 @@
 #include <unistd.h>
 #include <time.h>
 
-#include <qpopupmenu.h>
+#include <kpopupmenu.h>
 #include <qtimer.h>
 
 #include <klocale.h>
@@ -23,6 +23,8 @@
 #include "amor.moc"
 #include "amorpm.h"
 #include "amorbubble.h"
+#include "amorwidget.h"
+#include "amordialog.h"
 #include "version.h"
 #include <kdebug.h>
 #include <X11/Xlib.h>
@@ -70,10 +72,14 @@ Amor::Amor() : QObject(), DCOPObject( "AmorIface" )
                 this, SLOT(slotStackingChanged()));
         connect(mWin, SIGNAL(windowChanged(WId)),
                 this, SLOT(slotWindowChange(WId)));
+        connect(mWin, SIGNAL(currentDesktopChanged(int)),
+                this, SLOT(slotDesktopChange(int)));
 
         mAmor = new AmorWidget();
         connect(mAmor, SIGNAL(mouseClicked(const QPoint &)),
                         SLOT(slotMouseClicked(const QPoint &)));
+        connect(mAmor, SIGNAL(dragged(const QPoint &, bool)),
+                        SLOT(slotWidgetDragged(const QPoint &, bool)));
         mAmor->resize(mTheme.maximumSize());
 
         mTimer = new QTimer(this);
@@ -148,6 +154,8 @@ void Amor::reset()
     mAmor = new AmorWidget();
     connect(mAmor, SIGNAL(mouseClicked(const QPoint &)),
                     SLOT(slotMouseClicked(const QPoint &)));
+    connect(mAmor, SIGNAL(dragged(const QPoint &, bool)),
+		    SLOT(slotWidgetDragged(const QPoint &, bool)));
     mAmor->resize(mTheme.maximumSize());
 }
 
@@ -172,17 +180,28 @@ bool Amor::readConfig()
         return false;
     }
 
-    const char *groups[] = { ANIM_BASE, ANIM_NORMAL, ANIM_FOCUS, ANIM_BLUR,
-                            ANIM_DESTROY, ANIM_SLEEP, ANIM_WAKE, 0 };
-
-    // Read all the standard animation groups
-    for (int i = 0; groups[i]; i++)
+    if ( !mTheme.isStatic() )
     {
-        if (mTheme.readGroup(groups[i]) == false)
-        {
-            KMessageBox::error(0, i18n("Error reading group: ") + groups[i]);
-            return false;
-        }
+	const char *groups[] = { ANIM_BASE, ANIM_NORMAL, ANIM_FOCUS, ANIM_BLUR,
+				ANIM_DESTROY, ANIM_SLEEP, ANIM_WAKE, 0 };
+
+	// Read all the standard animation groups
+	for (int i = 0; groups[i]; i++)
+	{
+	    if (mTheme.readGroup(groups[i]) == false)
+	    {
+		KMessageBox::error(0, i18n("Error reading group: ") + groups[i]);
+		return false;
+	    }
+	}
+    }
+    else
+    {
+	if ( mTheme.readGroup( ANIM_BASE ) == false )
+	{
+	    KMessageBox::error(0, i18n("Error reading group: ") + ANIM_BASE);
+	    return false;
+	}
     }
 
     // Get the base animation
@@ -245,16 +264,30 @@ void Amor::selectAnimation(State state)
             if (mTargetWin != None)
             {
                 mTargetRect = KWin::info(mTargetWin).frameGeometry;
-                if (mCurrAnim->frame())
-                {
-                    mPosition = (kapp->random() %
-                        (mTargetRect.width() - mCurrAnim->frame()->width())) +
-                         mCurrAnim->hotspot().x();
-                }
-                else
-                {
-                    mPosition = mTargetRect.width()/2;
-                }
+		if ( mTheme.isStatic() )
+		{
+		    if ( mConfig.mStaticPos < 0 )
+			mPosition = mTargetRect.width() + mConfig.mStaticPos;
+		    else
+			mPosition = mConfig.mStaticPos;
+		    if ( mPosition >= mTargetRect.width() )
+			mPosition = mTargetRect.width()-1;
+		    else if ( mPosition < 0 )
+			mPosition = 0;
+		}
+		else
+		{
+		    if (mCurrAnim->frame())
+		    {
+			mPosition = (kapp->random() %
+			    (mTargetRect.width() - mCurrAnim->frame()->width())) +
+			     mCurrAnim->hotspot().x();
+		    }
+		    else
+		    {
+			mPosition = mTargetRect.width()/2;
+		    }
+		}
             }
             else
             {
@@ -329,7 +362,7 @@ void Amor::restack()
         return;
     }
 
-    kdDebug(10000) << "restacking" << endl;
+//    kdDebug(10000) << "restacking" << endl;
 
     Window sibling = mTargetWin;
     Window dw, parent = None, *wins;
@@ -360,26 +393,6 @@ void Amor::restack()
                      &values);
 }
 
-//---------------------------------------------------------------------------
-//
-QRect Amor::windowGeometry(WId win)
-{
-    QRect rect;
-    XWindowAttributes attr;
-    if (XGetWindowAttributes(qt_xdisplay(), win, &attr))
-    {
-        int x, y;
-        Window child;
-        XTranslateCoordinates(qt_xdisplay(), win, qt_xrootwin(),
-                              0, 0, &x, &y, &child);
-        rect.setRect(x, y, attr.width, attr.height);
-    }
-
-    kdDebug(10000) << "Window geometry: " << rect.x() << ", " << rect.y() << ", " << rect.width() << " x " << rect.height() << endl;
-
-    return rect;
-}
-
 //---------------------------------------------------------------------------
 //
 // The user clicked on our animation.
@@ -396,7 +409,8 @@ void Amor::slotMouseClicked(const QPoint &pos)
 
     if (!mMenu)
     {
-        mMenu = new QPopupMenu();
+        mMenu = new KPopupMenu();
+	mMenu->insertTitle("Amor"); // I really don't want this i18n'ed
         mMenu->insertItem(i18n("&Options..."), this, SLOT(slotConfigure()));
         mMenu->insertItem(i18n("&About..."), this, SLOT(slotAbout()));
         mMenu->insertSeparator();
@@ -444,7 +458,8 @@ void Amor::slotCursorTimeout()
 //
 void Amor::slotTimeout()
 {
-    mPosition += mCurrAnim->movement();
+    if (!mTheme.isStatic())
+	mPosition += mCurrAnim->movement();
     mAmor->setPixmap(mCurrAnim->frame());
     mAmor->move(mPosition + mTargetRect.x() - mCurrAnim->hotspot().x(),
                  mTargetRect.y() - mCurrAnim->hotspot().y() + mConfig.mOffset);
@@ -470,7 +485,10 @@ void Amor::slotTimeout()
         }
     }
 
-    mTimer->start(mCurrAnim->delay(), true);
+    if (mTheme.isStatic())
+	mTimer->start(mState == Normal ? 10000 : 0, true);
+    else
+	mTimer->start(mCurrAnim->delay(), true);
 
     if (!mCurrAnim->next())
     {
@@ -535,13 +553,42 @@ void Amor::slotAbout()
     KMessageBox::about(0, about, i18n("About Amor"));
 }
 
+//---------------------------------------------------------------------------
+//
+// Widget dragged
+//
+void Amor::slotWidgetDragged( const QPoint &delta, bool release )
+{
+    if (mCurrAnim->frame())
+    {
+	int newPosition = mPosition + delta.x();
+	if (mCurrAnim->totalMovement() + newPosition > mTargetRect.width())
+	    newPosition = mTargetRect.width() - mCurrAnim->totalMovement();
+	else if (mCurrAnim->totalMovement() + newPosition < 0)
+	    newPosition = -mCurrAnim->totalMovement();
+	mPosition = newPosition;
+        mAmor->move(mPosition + mTargetRect.x() - mCurrAnim->hotspot().x(),
+                 mAmor->y());
+
+	if ( mTheme.isStatic() && release ) {
+	    // static animations save the new position as preferred.
+	    int savePos = mPosition;
+	    if ( savePos > mTargetRect.width()/2 )
+		savePos -= (mTargetRect.width()+1);
+
+	    mConfig.mStaticPos = savePos;
+	    mConfig.write();
+	}
+    }
+}
+
 //---------------------------------------------------------------------------
 //
 // Focus changed to a different window
 //
 void Amor::slotWindowActivate(WId win)
 {
-    kdDebug(10000) << "Window activated" << endl;
+//    kdDebug(10000) << "Window activated:" << win << endl;
 
     mTimer->stop();
     mNextTarget = win;
@@ -560,7 +607,8 @@ void Amor::slotWindowActivate(WId win)
     else if (mNextTarget)
     {
         // We are setting focus to a new window
-        mState = Focus;
+        if (mState != Focus )
+	    selectAnimation(Focus);
         mTimer->start(0, true);
     }
     else
@@ -577,7 +625,7 @@ void Amor::slotWindowActivate(WId win)
 //
 void Amor::slotWindowRemove(WId win)
 {
-    kdDebug(10000) << "Window removed" << endl;
+//    kdDebug(10000) << "Window removed" << endl;
     if (win == mTargetWin)
     {
         // This is an active event that affects the target window
@@ -595,7 +643,7 @@ void Amor::slotWindowRemove(WId win)
 //
 void Amor::slotStackingChanged()
 {
-    kdDebug(10000) << "Stacking changed" << endl;
+//    kdDebug(10000) << "Stacking changed" << endl;
 
     // This is an active event that affects the target window
     time(&mActiveTime);
@@ -626,7 +674,7 @@ void Amor::slotWindowChange(WId win)
     if (info.isIconified() ||
         info.mappingState == NET::Withdrawn)
     {
-        kdDebug(10000) << "Iconic" << endl;
+//        kdDebug(10000) << "Iconic" << endl;
         // The target window has been iconified
         selectAnimation(Destroy);
         mTargetWin = None;
@@ -636,14 +684,24 @@ void Amor::slotWindowChange(WId win)
     else
     {
         // The size or position of the window has changed.
-//        mTargetRect = windowGeometry(mTargetWin);
         mTargetRect = info.frameGeometry;
 
         // make sure the animation is still on the window.
         if (mCurrAnim->frame())
         {
             hideBubble();
-            if (mPosition > mTargetRect.width() -
+	    if (mTheme.isStatic())
+	    {
+		if ( mConfig.mStaticPos < 0 )
+		    mPosition = mTargetRect.width() + mConfig.mStaticPos;
+		else
+		    mPosition = mConfig.mStaticPos;
+		if ( mPosition >= mTargetRect.width() )
+		    mPosition = mTargetRect.width()-1;
+		else if ( mPosition < 0 )
+		    mPosition = 0;
+	    }
+            else if (mPosition > mTargetRect.width() -
                     (mCurrAnim->frame()->width() - mCurrAnim->hotspot().x()))
             {
                 mPosition = mTargetRect.width() -
@@ -656,6 +714,20 @@ void Amor::slotWindowChange(WId win)
     }
 }
 
+//---------------------------------------------------------------------------
+//
+// Changed to a different desktop
+//
+void Amor::slotDesktopChange(int)
+{
+//    kdDebug(10000) << "Desktop change" << endl;
+    mNextTarget = None;
+    mTargetWin = None;
+    selectAnimation( Normal );
+    mTimer->stop();
+    mAmor->hide();
+}
+
 //===========================================================================
 
 AmorSessionWidget::AmorSessionWidget()
